[data-md-color-scheme="slate"] {
  --md-primary-fg-color: #30B06F;
  --md-accent-fg-color: #6791E0;
}

[data-md-color-scheme="default"] {
  --md-primary-fg-color: #30B06F;
  --md-accent-fg-color: #6791E0;
}

.md-nav__item--nested > .md-nav__link {
  color: #30B06F !important;
  font-weight: 700;
}
